#include <iostream>
#include <stdexcept>
#include <stdlib.h>
#include <wiringPi.h>

using namespace std;

float getDistanceCm(int TRIG, int ECHO) {
  long ping = 0;
  long pong = 0;
  float distance = 0;
  long timeout = 500000; // 0.5 сек ~ 171 м

  if (wiringPiSetup() == -1) {
    throw logic_error("WiringPi Setup ERROR");
  }

  pinMode(TRIG, OUTPUT);
  pinMode(ECHO, INPUT);

  // Trigger the ping.
  digitalWrite(TRIG, HIGH);
  delayMicroseconds(12);
  digitalWrite(TRIG, LOW);

  // Wait for ping response, or timeout.
  while (digitalRead(ECHO) == LOW && micros() < timeout);

  // Cancel on timeout.
  if (micros() > timeout) {
      throw logic_error("Out of range");
  }

  ping = micros();

  // Wait for pong response, or timeout.
  while (digitalRead(ECHO) == HIGH && micros() < timeout);

  // Cancel on timeout.
  if (micros() > timeout) {
      throw logic_error("Out of range");
  }

  pong = micros();

  // Convert ping duration to distance.
  distance = (float) (pong - ping) * 0.017150;

  return distance;
}

int main() {
  int TRIG = 4;
  int ECHO = 1;

  cout << "Подключение дальномера HC-SR04 к Orange Pi PC." << endl;

  while(true) {
    try {
      cout << "растояние = " << getDistanceCm(TRIG, ECHO) << "см" << endl;
    } catch(exception& e) {
      cout << e.what() << endl;
    }
    delay(500);
  }
  return 0;
}
